@@ -2,28 +2,27 @@ from bravado.exception import HTTPError
 
 from django.contrib.auth.models import User
 from django.db import models, transaction
-from django.utils.translation import gettext_lazy as _
+from django.db.models import Case, Q, Value, When
 from django.utils.timezone import now
+from django.utils.translation import gettext_lazy as _
+from esi.models import Token
 
 from allianceauth.authentication.models import CharacterOwnership
 from allianceauth.eveonline.models import EveCharacter
 from allianceauth.eveonline.providers import ObjectNotFound
 from allianceauth.notifications import notify
 from allianceauth.services.hooks import get_extension_logger
-
 from app_utils.helpers import chunks
 from app_utils.logging import LoggerAddTag
-from esi.models import Token
 
 from . import __title__
 from .app_settings import (
-    STANDINGS_API_CHARID,
-    SR_OPERATION_MODE,
     SR_NOTIFICATIONS_ENABLED,
+    SR_OPERATION_MODE,
+    STANDINGS_API_CHARID,
 )
 from .helpers.esi_fetch import esi_fetch
 
-
 logger = LoggerAddTag(get_extension_logger(__name__), __title__)
 
 
@@ -204,15 +203,38 @@ class _ContactsWrapper:
             self.alliance.append(self.Contact(contact, self.allianceLabels, name_info))
 
 
+class AbstractStandingsRequestQuerySet(models.QuerySet):
+    def annotate_is_pending(self) -> models.QuerySet:
+        return self.annotate(
+            is_pending_annotated=Case(
+                When(Q(action_date__isnull=True) & Q(is_effective=False), then=True),
+                default=Value(False),
+                output_field=models.BooleanField(),
+            )
+        )
+
+    def annotate_is_actioned(self) -> models.QuerySet:
+        return self.annotate(
+            is_actioned_annotated=Case(
+                When(Q(action_date__isnull=False) & Q(is_effective=False), then=True),
+                default=Value(False),
+                output_field=models.BooleanField(),
+            )
+        )
+
+
 class AbstractStandingsRequestManager(models.Manager):
+    def get_queryset(self) -> models.QuerySet:
+        return AbstractStandingsRequestQuerySet(self.model, using=self._db)
+
     def process_requests(self) -> None:
         """Process all the Standing requests/revocation objects"""
         from .models import (
+            AbstractStandingsRequest,
             ContactSet,
             EveEntity,
             StandingRequest,
             StandingRevocation,
-            AbstractStandingsRequest,
         )
 
         if self.model == AbstractStandingsRequest:
@@ -333,22 +355,19 @@ class AbstractStandingsRequestManager(models.Manager):
 
         returns True if a request is pending API confirmation, False otherwise
         """
-        return (
-            self.filter(contact_id=contact_id)
-            .exclude(action_date__isnull=True)
-            .filter(is_effective=False)
-            .exists()
-        )
+        return self.filter(
+            contact_id=contact_id, action_date__isnull=False, is_effective=False
+        ).exists()
 
     def has_effective_request(self, contact_id: int) -> bool:
         """return True if an effective request exists for given contact_id,
         else False
         """
-        return self.filter(contact_id=contact_id).filter(is_effective=True).exists()
+        return self.filter(contact_id=contact_id, is_effective=True).exists()
 
     def pending_requests(self) -> models.QuerySet:
         """returns all pending requests for this class"""
-        return self.filter(action_date__isnull=True).filter(is_effective=False)
+        return self.filter(action_date__isnull=True, is_effective=False)
 
 
 class StandingRequestManager(AbstractStandingsRequestManager):
